from functools import reduce
from json.decoder import JSONDecodeError

import requests
from requests.exceptions import ConnectionError


def post_groupme_message(msg, bot_id):
    requests.post(
        "https://api.groupme.com/v3/bots/post",
        json={
            "bot_id": bot_id,
            "text": msg
        },
        timeout=10
    )


def upload_image(key, content):
    try:
        return True, requests.post(
            "https://api.imgur.com/3/image",
            headers={
                "Authorization": f"Client-ID {key}",
            },
            data=content
        ).json()["data"]["link"]
    except ConnectionError as ce:
        return False, {
            "explain": "Could not reach image service.",
            "exception": ce
        }
    except (KeyError, JSONDecodeError) as e:
        return False, {
            "explain": "Could not parse response from image service.",
            "exception": e
        }


def find_all_subclasses(baseClass):
    def get_subclasses(clazz):
        s = set(clazz.__subclasses__())
        return s | reduce(set.union, (get_subclasses(sc) for sc in s), set())
    return get_subclasses(baseClass)
